@@ -1,6 +1,7 @@
 import json
 import logging
 import os
+import shlex
 import subprocess
 import sys
 from typing import Any, Dict, List, NamedTuple, Optional, Tuple
@@ -19,47 +20,48 @@ class IdfApp(App):
     Idf App class
 
     Attributes:
-        app_path (str): App path
-        binary_path (str): binary file path
         elf_file (str): elf file path
         flash_args (dict[str, Any]): dict of flasher_args.json
         flash_files (list[FlashFile]): list of (offset, file path, encrypted) of files need to be flashed in
         flash_settings (dict[str, Any]): dict of flash settings
     """
 
-    FLASH_ARGS_FILENAME = 'flasher_args.json'
+    FLASH_ARGS_FILENAME = 'flash_args'
+    FLASH_ARGS_JSON_FILENAME = 'flasher_args.json'
 
     def __init__(
         self,
-        app_path: Optional[str] = None,
-        build_dir: Optional[str] = None,
         part_tool: Optional[str] = None,
         **kwargs,
     ):
-        """
-        Args:
-            app_path: App path
-            build_dir: Build directory
-            part_tool: Partition tool path
-        """
-        super().__init__(app_path, build_dir, **kwargs)
+        super().__init__(**kwargs)
+
+        # Optional info
+        self._sdkconfig = None
+        self._target = None
+        # the partition table is used for nvs
+        self._parttool = part_tool
+        self._partition_table = None
+
         if not self.binary_path:
             logging.debug('Binary path not specified, skipping parsing app...')
             return
 
         # Required if binary path exists
         self.elf_file = self._get_elf_file()
-        self.bin_file = self._get_bin_file()
 
-        self.flash_args, self.flash_files, self.flash_settings = self._parse_flash_args()
+        # loadable elf file skip the rest of these
+        if self.sdkconfig.get('APP_BUILD_TYPE_ELF_RAM'):
+            self.is_loadable_elf = True
+        else:
+            self.is_loadable_elf = False
 
-        # Optional info
-        self._sdkconfig = None
-        self._target = None
+        self.bin_file = None
+        self.flash_args, self.flash_files, self.flash_settings = {}, [], {}
 
-        # the partition table is used for nvs
-        self._parttool = part_tool
-        self._partition_table = None
+        if not self.is_loadable_elf:
+            self.bin_file = self._get_bin_file()
+            self.flash_args, self.flash_files, self.flash_settings = self._parse_flash_args_json()
 
     @property
     def parttool_path(self) -> str:
@@ -109,7 +111,7 @@ class IdfApp(App):
     def partition_table(self) -> Dict[str, Any]:
         """
         Returns:
-            dict generated by partition tool
+            partition table dict generated by the partition tool
         """
         if self._partition_table is not None:
             return self._partition_table
@@ -163,9 +165,7 @@ class IdfApp(App):
                 return os.path.realpath(os.path.join(self.binary_path, fn))
         raise ValueError(f'Bin file under {self.binary_path} not found')
 
-    def _parse_flash_args(
-        self,
-    ) -> Tuple[Dict[str, Any], List[FlashFile], Dict[str, str]]:
+    def _parse_flash_args(self) -> List[str]:
         flash_args_filepath = None
         for fn in os.listdir(self.binary_path):
             if fn == self.FLASH_ARGS_FILENAME:
@@ -176,6 +176,21 @@ class IdfApp(App):
             raise ValueError(f'{self.FLASH_ARGS_FILENAME} not found')
 
         with open(flash_args_filepath) as fr:
+            return shlex.split(fr.read().strip())
+
+    def _parse_flash_args_json(
+        self,
+    ) -> Tuple[Dict[str, Any], List[FlashFile], Dict[str, str]]:
+        flash_args_json_filepath = None
+        for fn in os.listdir(self.binary_path):
+            if fn == self.FLASH_ARGS_JSON_FILENAME:
+                flash_args_json_filepath = os.path.realpath(os.path.join(self.binary_path, fn))
+                break
+
+        if not flash_args_json_filepath:
+            raise ValueError(f'{self.FLASH_ARGS_JSON_FILENAME} not found')
+
+        with open(flash_args_json_filepath) as fr:
             flash_args = json.load(fr)
 
         def _is_encrypted(_flash_args: Dict[str, Any], _offset: int, _file_path: str):
@@ -212,15 +227,10 @@ class IdfApp(App):
             filepath: path to the file
 
         Returns:
-            sha256 appended to app
+            sha256 value appended to app
         """
-        from pytest_embedded_serial_esp.serial import EspSerial, EsptoolVersion
-
-        if EspSerial.ESPTOOL_VERSION == EsptoolVersion.V3:
-            from esptool import LoadFirmwareImage, hexify
-        else:
-            from esptool.bin_image import LoadFirmwareImage
-            from esptool.util import hexify
+        from esptool.bin_image import LoadFirmwareImage
+        from esptool.util import hexify
 
         image = LoadFirmwareImage(self.target, filepath)
         if image.append_digest:
