@@ -2,11 +2,11 @@ import hashlib
 import logging
 import os
 import tempfile
-from typing import Dict, Optional, TextIO, Union
+from typing import Optional, TextIO, Union
 
 import esptool
-from pytest_embedded.log import PexpectProcess
-from pytest_embedded_serial_esp.serial import EspSerial, EsptoolVersion
+from pytest_embedded.log import live_print_call
+from pytest_embedded_serial_esp.serial import EspSerial, EsptoolArgs
 
 from .app import IdfApp
 
@@ -23,21 +23,12 @@ class IdfSerial(EspSerial):
 
     def __init__(
         self,
-        pexpect_proc: PexpectProcess,
         app: IdfApp,
         target: Optional[str] = None,
-        beta_target: Optional[str] = None,
-        port: Optional[str] = None,
-        baud: int = EspSerial.DEFAULT_BAUDRATE,
-        esptool_baud: int = EspSerial.ESPTOOL_DEFAULT_BAUDRATE,
-        skip_autoflash: bool = False,
-        erase_all: bool = False,
-        port_app_cache: Dict[str, str] = None,
         confirm_target_elf_sha256: bool = False,
         erase_nvs: bool = False,
         **kwargs,
     ) -> None:
-        self._port_app_cache: Dict[str, str] = port_app_cache if port_app_cache is not None else {}
         self.app = app
         self.confirm_target_elf_sha256 = confirm_target_elf_sha256
         self.erase_nvs = erase_nvs
@@ -49,38 +40,27 @@ class IdfSerial(EspSerial):
             raise ValueError(f'Targets do not match. App target: {self.app.target}, Cmd target: {target}.')
 
         super().__init__(
-            pexpect_proc,
-            target or app.target,
-            beta_target,
-            port,
-            baud,
-            esptool_baud,
-            skip_autoflash,
-            erase_all,
+            target=target or app.target,
             **kwargs,
         )
 
     def _post_init(self):
-        if self.esp.serial_port in self._port_app_cache:
-            if self.app.binary_path == self._port_app_cache[self.esp.serial_port]:  # hit the cache
-                logging.debug('hit port-app cache: %s - %s', self.port, self.app.binary_path)
-                if self.confirm_target_elf_sha256:
-                    if self.is_target_flashed_same_elf():
-                        logging.info('Confirmed target elf file sha256 the same as your local one.')
-                        self.skip_autoflash = True
-                    else:
-                        logging.info('target elf file is different from your local one. Flash the binary again.')
-                        self.skip_autoflash = False
-                else:
-                    logging.info(
-                        'App is the same according to the session cache. '
-                        'you can use flag "--confirm-target-elf-sha256" to make sure '
-                        'that the target elf file is the same as your local one.'
-                    )
+        if self._meta and self._meta.hit_port_app_cache(self.port, self.app):
+            if self.confirm_target_elf_sha256:
+                if self.is_target_flashed_same_elf():
+                    logging.info('Confirmed target elf file sha256 the same as your local one.')
                     self.skip_autoflash = True
+                else:
+                    logging.info('target elf file is different from your local one. Flash the binary again.')
+                    self.skip_autoflash = False
+            else:
+                logging.info(
+                    'App is the same according to the session cache. '
+                    'you can use flag "--confirm-target-elf-sha256" to make sure '
+                    'that the target elf file is the same as your local one.'
+                )
+                self.skip_autoflash = True
 
-        logging.debug('set port-app cache: %s - %s', self.port, self.app.binary_path)
-        self._port_app_cache[self.port] = self.app.binary_path
         super()._post_init()
 
     def _start(self):
@@ -88,9 +68,39 @@ class IdfSerial(EspSerial):
             logging.info('Skipping auto flash...')
             super()._start()
         else:
-            self.flash()
+            if self.app.is_loadable_elf:
+                self.load_ram()
+            else:
+                self.flash()
 
-    @EspSerial.use_esptool
+    def load_ram(self) -> None:
+        if not self.app.is_loadable_elf:
+            raise ValueError('elf should be loadable elf')
+
+        live_print_call(
+            [
+                'esptool.py',
+                '--chip',
+                self.app.target,
+                'elf2image',
+                self.app.elf_file,
+                *self.app._parse_flash_args(),
+            ],
+            msg_queue=self._q,
+        )
+        live_print_call(
+            [
+                'esptool.py',
+                '--chip',
+                self.app.target,
+                '--no-stub',
+                'load_ram',
+                self.app.elf_file.replace('.elf', '.bin'),
+            ],
+            msg_queue=self._q,
+        )
+
+    @EspSerial.use_esptool()
     def flash(self) -> None:
         """
         Flash the `app.flash_files` to the dut
@@ -107,53 +117,44 @@ class IdfSerial(EspSerial):
         encrypt_files = [(file.offset, open(file.file_path, 'rb')) for file in self.app.flash_files if file.encrypted]
 
         nvs_file = None
-        if self.erase_nvs:
-            address = self.app.partition_table['nvs']['offset']
-            size = self.app.partition_table['nvs']['size']
-            nvs_file = tempfile.NamedTemporaryFile(delete=False)
-            nvs_file.write(b'\xff' * size)
-            if not isinstance(address, int):
-                address = int(address, 0)
-
-            if self.app.flash_settings['encrypt']:
-                encrypt_files.append((address, open(nvs_file.name, 'rb')))
-            else:
-                flash_files.append((address, open(nvs_file.name, 'rb')))
-
-        # fake flasher args object, this is a hack until
-        # esptool Python API is improved
-        class FlashArgs(object):
-            def __init__(self, attributes):
-                for key, value in attributes.items():
-                    self.__setattr__(key, value)
-
-        # write_flash expects the parameter encrypt_files to be None and not
-        # an empty list, so perform the check here
-        default_kwargs = {
-            'addr_filename': flash_files,
-            'encrypt_files': encrypt_files or None,
-            'no_stub': False,
-            'compress': True,
-            'verify': False,
-            'ignore_flash_encryption_efuse_setting': False,
-            'erase_all': False,
-        }
-
-        if self.ESPTOOL_VERSION == EsptoolVersion.V4:
-            default_kwargs['force'] = False
-
-        if self.erase_all:
-            default_kwargs['erase_all'] = True
-
-        default_kwargs.update(self.app.flash_settings)
-        default_kwargs.update(self.app.flash_args.get('extra_esptool_args', {}))
-        args = FlashArgs(default_kwargs)
-
         try:
+            if self.erase_nvs:
+                address = self.app.partition_table['nvs']['offset']
+                size = self.app.partition_table['nvs']['size']
+                nvs_file = tempfile.NamedTemporaryFile(delete=False)
+                nvs_file.write(b'\xff' * size)
+                if not isinstance(address, int):
+                    address = int(address, 0)
+
+                if self.app.flash_settings['encrypt']:
+                    encrypt_files.append((address, open(nvs_file.name, 'rb')))
+                else:
+                    flash_files.append((address, open(nvs_file.name, 'rb')))
+
+            # write_flash expects the parameter encrypt_files to be None and not
+            # an empty list, so perform the check here
+            default_kwargs = {
+                'addr_filename': flash_files,
+                'encrypt_files': encrypt_files or None,
+                'no_stub': False,
+                'compress': True,
+                'verify': False,
+                'ignore_flash_encryption_efuse_setting': False,
+                'erase_all': False,
+                'force': False,
+            }
+
+            default_kwargs.update(self.app.flash_settings)
+            default_kwargs.update(self.app.flash_args.get('extra_esptool_args', {}))
+            args = EsptoolArgs(**default_kwargs)
+
             self.stub.change_baud(self.esptool_baud)
             esptool.detect_flash_size(self.stub, args)
             esptool.write_flash(self.stub, args)
             self.stub.change_baud(self.baud)
+
+            if self._meta:
+                self._meta.set_port_app_cache(self.port, self.app)
         finally:
             if nvs_file:
                 nvs_file.close()
@@ -166,7 +167,7 @@ class IdfSerial(EspSerial):
             for (_, f) in encrypt_files:
                 f.close()
 
-    @EspSerial.use_esptool
+    @EspSerial.use_esptool()
     def dump_flash(
         self,
         partition: Optional[str] = None,
@@ -208,7 +209,7 @@ class IdfSerial(EspSerial):
         else:
             return content
 
-    @EspSerial.use_esptool
+    @EspSerial.use_esptool()
     def erase_partition(self, partition_name: str) -> None:
         """
         Erase the partition provided
@@ -227,15 +228,7 @@ class IdfSerial(EspSerial):
         else:
             raise ValueError(f'partition name "{partition_name}" not found in app partition table')
 
-    @EspSerial.use_esptool
-    def erase_flash(self) -> None:
-        """
-        Erase the complete flash
-        """
-        logging.info('Erasing the flash')
-        self.stub.erase_flash()
-
-    @EspSerial.use_esptool
+    @EspSerial.use_esptool()
     def read_flash_elf_sha256(self) -> bytes:
         """
         Read the sha256 digest of the flashed elf file
